package org.apache.lucene.search;

/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *    "Apache Lucene" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    "Apache Lucene", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

import java.util.BitSet;
import java.io.IOException;

import org.apache.lucene.index.Term;
import org.apache.lucene.index.TermDocs;
import org.apache.lucene.index.TermEnum;
import org.apache.lucene.index.IndexReader;

/** 
 * A Filter that restricts search results to a exactly match a string
 * or start with prefix
 *
 * @author Che, Dong <chedong@bigfoot.com>
 * $Header: /home/cvsroot/lucene_ext/src/org/apache/lucene/search/StringFilter.java,v 1.1.1.1 2002/09/22 19:36:08 chedong Exp $
 */
public final class StringFilter extends Filter {
	String field = null;			//compare field
	String s = null ;		//compare string
	boolean strict = true;	//true: strict mach,false: prefix match 
		
	/**
	* Constructs a filter for field <code>f</code> matching <code>
	* compare</code> String
	*/
	public StringFilter(String f, String compare) {
		field = f;
		s = compare;
	}
	
	
	public StringFilter(String f, String compare, boolean str) {
		field = f;
		s = compare;
		strict = str;
	}
	
	/** Constructs a filter for field <code>f</code> matching string start with
	<code>prefix</code>. */
	public static StringFilter Like(String field, String prefix) {
		StringFilter result = new StringFilter(field, prefix,false);    
		return result;
	}
	
	/** 
	* Returns a BitSet with true for documents which should be permitted in
	* search results, and false for those that should not. 
	*/
	final public BitSet bits(IndexReader reader) throws IOException {
		BitSet bits = new BitSet(reader.maxDoc());
		TermEnum enum = reader.terms();
		try {	
			//exactly match
			if (strict == true) {				
				Term stop = new Term(field, s);				
				while (enum.next()) {							
					if (enum.term().compareTo(stop) == 0) {					
						TermDocs termDocs = reader.termDocs(enum.term());
						try {
							while (termDocs.next())
								bits.set(termDocs.doc());
						} finally {
							termDocs.close();
						}
					}
				}
			}
			//prefix match
			else {			
				while (enum.next()) {			
					if ( enum.term().field().equals(field) && enum.term().text().startsWith(s)) {
						TermDocs termDocs = reader.termDocs(enum.term());
						try {
							while (termDocs.next())
								bits.set(termDocs.doc());
						} finally {
							termDocs.close();
						}
					}						
				}
			}
		} finally {
			enum.close();
		}
		return bits;
	}
	
	public final String toString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append(field);
		buffer.append(":");
		if (strict == true)
			buffer.append("is " + s);
		else
			buffer.append("is prefix match " + s);    
		return buffer.toString();
	}
}